%{
/*
// THIS FILE IS COPYRIGHT 1994 BY TELENOR RESEARCH, NORWAY
//
//                    ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
*/

/* Programming : Jon Stephenson von Tetzchner */

#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include "trans.h"

/*
 * Avoiding stack overflows...
 * Jan van der Steen (jansteen@cwi.nl)
 */

#define YYMAXDEPTH 4000
 

%}

%token MIFSTART
%token MIFEND
%token PAGESTART
%token PAGETYPE
%token IMPORTOBJECTSTART
%token IMPORTOBFILE
%token IMPORTOBFILEDI
%token BRECT
%token ANGLE
%token FLIPLR
%token IMPORTOBJECTEND
%token PAGEEND
%token PGFCATALOGSTART
%token PGFCATALOGEND
%token PARASTART
%token PGFTAG
%token PGFALIGNMENT
%token PGFTOPSEPARATOR
%token PGFBOTSEPARATOR
%token PGFSTART
%token PGFNUMBERFONT
%token PGFFONTSTART
%token PGFFONTEND
%token STRING
%token PGFEND
%token PGFNUMSTRING
%token PARALINE
%token PARAEND
%token FONTCATALOGSTART
%token FONTCATALOGEND
%token FTAG
%token FONTSTART
%token FFAMILY
%token FWEIGHT
%token FANGLE
%token FUNDERLINE
%token FSUPERSCRIPT
%token FSUBSCRIPT
%token FONTEND
%token TBLCATALOGSTART
%token TBLCATALOGEND
%token FNOTEUSE
%token FNOTESTART
%token FNOTEEND
%token XREFSTART
%token XREFSRCTEXT
%token XREFSRCFILE
%token XREFEND
%token AFRAMESSTART
%token AFRAMESEND
%token FRAMESTART
%token INTFRAMESTART
%token FRAMEEND
%token FRAMETYPE
%token AFRAMEUSE
%token TABLESSTART
%token TABLESTART
%token TABLEID
%token TABLEFORMATSTART
%token TABLEFORMATEND
%token TABLENUMCOLUMNS
%token TABLETITLESTART
%token TABLETITLEEND
%token TABLEHEADSTART
%token TABLEHEADEND
%token TABLEFOOTSTART
%token TABLEFOOTEND
%token TABLEBODYSTART
%token TABLEROWSTART
%token TABLECELLSTART
%token TABLECELLCOLUMNS
%token TABLECELLEND
%token TABLEROWEND
%token TABLEBODYEND
%token TABLEEND
%token TABLESEND
%token TABLEUSE
%token MARKERTYPE
%token MARKERTEXT
%token TEXTFLOWSTART
%token TEXTFLOWTAG
%token TEXTRECTID
%token TEXTFLOWEND
%token TEXTRECTSTART
%token TEXTRECTEND
%token SPECIALCHAR

%token VARISTART
%token VARINAME
%token VARIEND
%token VARIFORMSSTART
%token VARIFORMSTART
%token VARIDEF
%token VARIFORMEND
%token VARIFORMSEND


%%
miffile		:	MIFSTART {StartPrint();} headitems pages textflows MIFEND {EndPrint();}
                ;	

headitems	:	headitems headitem
                |	headitem
                ;

headitem	:	pgfcatalog
                |	fontcatalog
                |	tblcatalog
                |	aframes
                |	tablesection
                |	variformatsection
                |	skip
                ;

pgfcatalog	:	pgfcatalogstart pgfs pgfcatalogend
                |	pgfcatalogstart pgfcatalogend
                ;

pgfcatalogstart	:	PGFCATALOGSTART {PgfCatalogStart();}
                ;

pgfcatalogend	:	PGFCATALOGEND {PgfCatalogEnd();}
                ;

fontcatalog	:	fontcatalogstart fonts fontcatalogend
                |	fontcatalogstart fontcatalogend
                ;

fontcatalogstart	:	FONTCATALOGSTART {FontCatalogStart();}
                ;

fontcatalogend	:	FONTCATALOGEND {FontCatalogEnd();}
                ;

tblcatalog	:	TBLCATALOGSTART tableformats TBLCATALOGEND
                |	TBLCATALOGSTART TBLCATALOGEND
                ;

tableformats	:	tableformats tableformat
                |	tableformat
                ;

tableformat	:	TABLEFORMATSTART skips TABLEFORMATEND
                ;

aframes		:	aframesstart frames aframesend
                |	aframesstart aframesend
                ;

aframesstart	:	AFRAMESSTART {AFramesStart();}
                ;

aframesend	:	AFRAMESEND {AFramesEnd();}
                ;

frames		:	frames frame
                |	frame
                ;

frame	        :	framestart framecontents frameend
                ;

framestart	:	FRAMESTART {FrameStart();}
                ;

frameend        :	FRAMEEND {FrameEnd();}
                ;

framecontents	:	framecontents framecontent
                |	framecontent
                ;

framecontent 	:	frameitem
                |	FRAMETYPE {SetFrameType();}
                |	INTFRAMESTART framecontents FRAMEEND
                ;

inlineframe	:	inlineframestart intframecontents inlineframeend
                |	inlineframestart inlineframeend
                ;

inlineframestart	:	INTFRAMESTART {InlineFrameStart();}
                ;

inlineframeend	:	FRAMEEND {InlineFrameEnd();}
                ;

intframecontents	:	intframecontents intframecontent
                |	intframecontent
                ;

intframecontent	:	frameitem {PrintFrameItem();}
                |	FRAMETYPE {PrintFrameItem();}
                |	INTFRAMESTART {PrintFrameItem();} intframecontents FRAMEEND {PrintFrameItem();}
                ;

frameitem	: PARASTART | PGFTAG | PGFNUMBERFONT | PGFNUMSTRING | PARALINE 
                | STRING | PARAEND | FNOTEUSE | FNOTESTART | FNOTEEND
                | XREFSTART | XREFSRCTEXT | XREFSRCFILE | XREFEND | AFRAMEUSE
                | TABLEUSE | MARKERTYPE | MARKERTEXT | TEXTFLOWSTART
                | TEXTFLOWTAG | TEXTRECTID | TEXTFLOWEND
                | TABLEROWSTART | TABLECELLSTART | TABLECELLEND | TABLEROWEND
                | FONTSTART | FTAG | FFAMILY | FWEIGHT | FANGLE | FUNDERLINE
                | FSUPERSCRIPT | FSUBSCRIPT | SPECIALCHAR | FONTEND
                | PGFFONTSTART | PGFFONTEND
                | IMPORTOBJECTSTART | IMPORTOBFILE | IMPORTOBFILEDI
                | BRECT | ANGLE | FLIPLR | IMPORTOBJECTEND
                | textrectstart | textrectend | variname
                ;

textrect	:	textrectstart textrectitems textrectend
                ;

textrectstart	:	TEXTRECTSTART {TextRectStart();}
                ;

textrectend	:	TEXTRECTEND {TextRectEnd();}
                ;

textrectitems	:	textrectitems textrectitem
                |	textrectitem
                ;

textrectitem	:	BRECT
                |	ANGLE
                |	FLIPLR
                ;

tablesection	:	tablesstart tables tablesend
                |	tablesstart tablesend
                ; 

tablesstart	:	TABLESSTART {TablesStart();}
                ; 

tablesend	:	TABLESEND {TablesEnd();}
                ; 

tables		:	tables table
                |	table
                ;

table		:	TABLESTART {TableStart();} tableitems TABLEEND {TableEnd();}
                ;

tableitems	:	tableitems tableitem
                |	tableitem
                ;

tableitem	:	tableid
                |	tableformat
                |	tabletitle
                |	tablehead
                |	tablenumcolumns
                |	tablebody
                |	tablefoot
                ;

tableid		:	TABLEID {GetTableID();}
                ;

tablenumcolumns	:	TABLENUMCOLUMNS {GetTableNumColumns();}
                ;

tabletitle	:	TABLETITLESTART {TableTitleStart();} paragraphs TABLETITLEEND {TableTitleEnd();} 
                ;

tablehead	:	TABLEHEADSTART {TableHeadStart();} tablerows TABLEHEADEND {TableHeadEnd();} 
                ;

tablebody	:	TABLEBODYSTART {TableBodyStart();} tablerows TABLEBODYEND {TableBodyEnd();} 
                ;

tablefoot	:	TABLEFOOTSTART {TableFootStart();} tablerows TABLEFOOTEND {TableFootEnd();} 
                ;

tablerows	:	tablerows tablerow
                |	tablerow
                ;

tablerow	:	TABLEROWSTART {TableRowStart();} tablecells TABLEROWEND {TableRowEnd();}
                ;

tablecells	:	tablecells tablecell
                |	tablecell
                ;

tablecell	:	tablecellstart tablecellitems tablecellend
                |	tablecellstart tablecellend
                ;

tablecellstart	:	TABLECELLSTART {TableCellStart();}
                ;

tablecellend	:	TABLECELLEND {TableCellEnd();}
                ;

tablecellitems	:	tablecellitems tablecellitem
                |	tablecellitem
                ;

tablecellitem	:	tablecellcolumns
                |	fnote
                |	paragraph
                ;

tablecellcolumns	:	TABLECELLCOLUMNS {TableCellColumns();}
		;

pages		:	pages page
                |	page
                ;

page		:	pagestart pageitems pageend
                ;

pagestart	:	PAGESTART {PageStart();}
                ;

pageend		:	PAGEEND {PageEnd();}
                ;

pageitems	:	pageitems pageitem
                |	pageitem
                ;

pageitem	:	PAGETYPE {PageType();}
                |	STRING
                |	font
                |	textrect
                |	SPECIALCHAR
                |	importobject
                |	inlineframe
                |	variname
                |	BRECT
                |	ANGLE
                |	FLIPLR
                ;

importobject	:	importobjectstart importobjectitems importobjectend
                |	importobjectstart importobjectend
                ;

importobjectstart	:	IMPORTOBJECTSTART {ImportObjectStart();}
                ;

importobjectend	:	IMPORTOBJECTEND {ImportObjectEnd();}
                ;

importobjectitems	:	importobjectitems importobjectitem
                |	importobjectitem
                ;

importobjectitem	:	importobfile
                |	importobfiledi
                |	brect
                |	angle
                |	fliplr
                ;

importobfile	:	IMPORTOBFILE {ImportObFile();}
                ;

importobfiledi	:	IMPORTOBFILEDI {ImportObFileDI();}
                ;

brect		:	BRECT {BRect();}
                ;

angle		:	ANGLE {Angle();}
                ;

fliplr		:	FLIPLR {FlipLR();}
                ;

textflows	:	textflows textflow
                |	textflow
                ;

textflow      	:	textflowstart textflowitems textflowend
                |	textflowstart textflowend
                ;

textflowstart	:	TEXTFLOWSTART {TextFlowStart();}
                ;

textflowend	:	TEXTFLOWEND {TextFlowEnd();}
                ;

textflowitems	:	textflowitems textflowitem
                |	textflowitem
                ;

textflowitem	:	textflowtag
                |	fnote
                |	paragraph
                ;

textflowtag     :	TEXTFLOWTAG {TextFlowTag();}
                ;

fnote		:	fnotestart paragraphs fnoteend
                ;

fnotestart	:	FNOTESTART {NewFootnoteStart();}
                ;

fnoteend	:	FNOTEEND {NewFootnoteEnd();}
                ;

paragraphs	:	paragraphs paragraph
                |	paragraph
                ;

paragraph 	:	parastart paraitems paraend
                |	parastart paraend
                ;

parastart 	:	PARASTART {ParaStart();}
                ;

paraend	 	:	PARAEND {ParaEnd();} 
                ;

paraitems	:	paraitems paraitem
                |	paraitem
                ;

paraitem	:	pgf
                |	pgftag
                |	pgfalignment
                |	pgfnumstring
                |	paraline
                ;

pgfs		:	pgfs pgf
                |	pgf
                ;

pgf	 	:	pgfstart pgfitems pgfend
                |	pgfstart pgfend
                ;

pgfstart 	:	PGFSTART {PgfStart();}
                ;

pgfend	 	:	PGFEND {PgfEnd();} 
                ;

pgfitems	:	pgfitems pgfitem
                |	pgfitem
                ;

pgfitem		:	pgftag
                |	pgfalignment
                |	pgftopseparator
                |	pgfbotseparator
                |	pgffont
                |	pgfnumberfont
                ;

pgftag		:	PGFTAG {ParagraphTag();}
                ;

pgfalignment	:	PGFALIGNMENT {ParagraphAlignment();}
                ;

pgftopseparator	:	PGFTOPSEPARATOR {ParagraphTopSeparator();}
                ;

pgfbotseparator	:	PGFBOTSEPARATOR {ParagraphBottomSeparator();}
                ;

pgffont		:	PGFFONTSTART fonttag fontitems PGFFONTEND
                ;

pgfnumberfont	:	PGFNUMBERFONT {ParagraphNumberFont();}
                ;

pgfnumstring	:	PGFNUMSTRING {ParagraphNumberString();}
                ;

paralines	:	paralines paraline
                |	paraline
                ;

paraline	:	PARALINE {HandleParaLine();} paraconts
                |	PARALINE {HandleParaLine();}
                ;

paraconts	:	paraconts paracont
                |	paracont
                ;

paracont	:	TEXTRECTID {PrintTextRectID();}
                |	string
                ;

strings		:	strings string
                |	string
                ;

string		:	STRING {PrintString();} 
                |	FNOTEUSE {InsertFootnote();} 
                |	AFRAMEUSE {AFrameUse();} 
                |	TABLEUSE {TableUse();} 
                |	MARKERTEXT {SetMarker();}
                |	MARKERTYPE {SetMarkerType();}
                |	SPECIALCHAR {InsertSpecialChar();}
                |	xref 
                |	font
                |	variname 
                ;

xref		:	xrefstart xrefitems strings paralines xrefend
                |	xrefstart xrefitems strings xrefend
                |	xrefstart xrefitems xrefend
                |	xrefstart xrefend
                ;

xrefstart	:	XREFSTART {InsertReferenceStart();} 
                ;

xrefitems	:	xrefitems xrefitem
                |	xrefitem
                ;

xrefitem	:	XREFSRCTEXT {SetReferenceNumber();}
                |	XREFSRCFILE {SetReferenceFile();}
                ;

xrefend 	:	XREFEND {InsertReferenceEnd();}
                ;

fonts		:	fonts font
                |	font
                ;

font	 	:	fontstart fontitems fontend
                ;

fontstart 	:	FONTSTART {FontStart();}
                ;

fontend	 	:	FONTEND {FontEnd();} 
                ;

fontitems	:	fontitems fontitem
                |	fontitem
                ;

fontitem	:	fonttag
                |	fontfamily
                |	fontweight
                |	fontangle
                |	fontunderline
                |	fontsubscript
                |	fontsuperscript
                ;

fonttag		:	FTAG {FontTag();}
                ;

fontfamily	:	FFAMILY {FontFamily();} 
                ;

fontweight	: 	FWEIGHT {FontWeight();} 
                ;

fontangle	:	FANGLE {FontAngle();} 
                ;

fontunderline	:	FUNDERLINE {FontUnderline();} 
                ;

fontsubscript	:	FSUBSCRIPT {FontSubscript();}
                ;
 
fontsuperscript	:	FSUPERSCRIPT {FontSuperscript();} 
                ;

skips		:	skips skip
                |	skip
                ;

skip		: IMPORTOBJECTSTART | IMPORTOBFILE | IMPORTOBFILEDI
                | BRECT | ANGLE | FLIPLR | IMPORTOBJECTEND
                | PAGEEND | PARASTART | PGFSTART | PGFTAG | PGFALIGNMENT 
                | PGFTOPSEPARATOR | PGFBOTSEPARATOR | PGFNUMBERFONT
                | PGFNUMSTRING | PARALINE | STRING | PGFEND | PARAEND
                | FNOTEUSE | FNOTESTART | SPECIALCHAR | FNOTEEND
                | XREFSTART | XREFSRCTEXT | XREFSRCFILE | XREFEND
                | AFRAMEUSE | TABLEUSE | MARKERTEXT | MARKERTYPE
                | TEXTFLOWTAG | TEXTRECTID | TEXTRECTSTART | TEXTRECTEND
                | TABLEROWSTART | TABLECELLSTART | TABLECELLEND | TABLEROWEND
                | FRAMESTART | FRAMEEND | FTAG | FFAMILY | FWEIGHT | FANGLE
                | FUNDERLINE | FSUPERSCRIPT | FSUBSCRIPT | INTFRAMESTART
                | FRAMETYPE | FONTSTART | FONTEND | PGFFONTSTART | PGFFONTEND
                | VARISTART | VARINAME | VARIEND
                ;

variformatsection	:	VARIFORMSSTART variformats VARIFORMSEND
                |	VARIFORMSSTART VARIFORMSEND
                ;

variformats	:	variformats variformat
                |	variformat
                ;

variformat	:	VARIFORMSTART VARINAME {SaveVariable();} VARIDEF {SaveVariableDef();} VARIFORMEND
                ;

variname        :	VARISTART VARINAME {NoteVariName();} VARIEND
                ;

%%

#include "trans.c"
#include "trans-code.tmp"
#include "trans_table-code.c"

/*
 * yyerror modified by
 * Jan van der Steen (jansteen@cwi.nl)
 */

yyerror(s)
char *s;
{
     if (s)
	 fprintf(stderr,"trans: ERROR in parsing. Got: \"%s\".\ntrans: Error: \"%s\"\n",
		 yytext, s);
     else
	fprintf(stderr, "trans: ERROR in parsing. Got %s .\n", yytext); 
}

